# -*- coding: utf-8 -*-
"""
    blohg.script
    ~~~~~~~~~~~~

    Module with the CLI script related stuff.

    :copyright: (c) 2010-2011 by Rafael Goncalves Martins
    :license: GPL-2, see LICENSE for more details.
"""

import os
import shutil
import sys

from flaskext.script import Command, Manager, Server
from mercurial import commands, ui, error
from pkg_resources import resource_filename, resource_listdir

from blohg import create_app


class InitRepo(Command):
    """initialize a blohg repo, using the default template."""

    def handle(self, app):
        repo_path = app.config.get('REPO_PATH')
        template_path = resource_filename('blohg', 'repo_template')
        template_rootfiles = resource_listdir('blohg', 'repo_template')

        initialized = False
        for f in template_rootfiles + ['.hg']:
            if os.path.exists(os.path.join(repo_path, f)):
                initialized = True

        if initialized:
            print >> sys.stderr, 'repository already initialized: %s' % repo_path
            return

        if not os.path.exists(repo_path):
            os.makedirs(repo_path)

        for f in template_rootfiles:
            full_path = os.path.join(template_path, f)
            if os.path.isdir(full_path):
                shutil.copytree(full_path, os.path.join(repo_path, f))
            elif os.path.isfile(full_path):
                shutil.copy2(full_path, os.path.join(repo_path, f))
            else:
                print >> sys.stderr, 'unrecognized file: %s' % full_path

        try:
            commands.init(ui.ui(), repo_path)
        except error, err:
            print >> sys.stderr, 'an error was occurred: %s' % err


def create_script():
    """Script object factory

    :param repo_path: the path to the mercurial repository.
    :return: the script object (Flask-Script' Manager instance).
    """

    script = Manager(create_app, with_default_commands=False)
    script.add_option('-r', '--repo-path', dest='repo_path', default=os.getcwd(),
        required=False)
    server = Server(use_debugger=True, use_reloader=True)
    server.description = 'runs the blohg local server.'
    script.add_command('runserver', server)
    script.add_command('initrepo', InitRepo())

    return script



# -*- coding: utf-8 -*-
"""
    blohg.views
    ~~~~~~~~~~~

    A generic Flask Module with all the views.

    :copyright: (c) 2010-2011 by Rafael Goncalves Martins
    :license: GPL-2, see LICENSE for more details.
"""

import math

from flask import Module, abort, current_app, make_response, render_template, \
     url_for
from werkzeug.contrib.atom import AtomFeed, FeedEntry

views = Module(__name__)


@views.route('/atom/')
@views.route('/atom/<path:tag>/')
@views.route('/rss/')  # dumb redirect to keep the compatibility
@views.route('/rss/<path:tag>/')  # dumb redirect to keep the compatibility
def atom(tag=None):
    """General purpose atom feed."""

    title = current_app.config['TITLE']
    if tag is not None:
        tags = tag.split('/')
        for _tag in tags:
            if _tag not in current_app.hg.tags:
                abort(404)
        title += u' » %s' % ' + '.join(tags)
        posts = current_app.hg.get_by_tag(tags)
    else:
        posts = current_app.hg.get_all(True)
    feed = AtomFeed(
        title=title,
        subtitle=current_app.config['TAGLINE'],
        url=url_for('views.home'),
        feed_url=url_for('views.atom', tag=tag),
        author=current_app.config['AUTHOR'],
        generator=('blohg', None, None)
    )
    for post in posts[:int(current_app.config['POSTS_PER_PAGE'])]:
        feed.add(
            FeedEntry(
                title=post.title,
                content=post.full_html,
                summary=post.abstract_html,
                id=url_for('views.content', slug=post.slug),
                url=url_for('views.content', slug=post.slug, _external=True),
                author=current_app.config['AUTHOR'],
                published=post.datetime,
                updated=post.datetime,
            )
        )
    return feed


@views.route('/<path:slug>/')
def content(slug):
    """Posts and static pages."""
    page = current_app.hg.get(slug)
    if page is None:
        abort(404)
    title = page.title
    if slug.startswith('post'):
        title = u'Post: %s' % page.title
    return render_template(
        '_posts.html',
        title=title,
        posts=[page],
        full_content=True,
    )


@views.route('/', defaults={'page': 1})
@views.route('/page/<int:page>/')
def home(page):
    """Page with the abstract of the posts. Part of the pagination."""
    current = int(page)
    pages = current_app.hg.get_all(True)
    ppp = int(current_app.config['POSTS_PER_PAGE'])
    num_pages = int(math.ceil(float(len(pages)) / ppp))
    init = int((current - 1) * ppp)
    end = int(current * ppp)
    return render_template(
        '_posts.html',
        posts=pages[init:end],
        full_content=False,
        pagination={
            'num_pages': num_pages,
            'current': page,
        }
    )


@views.route('/post/')
def post_list():
    """Page with a simple list of posts (link + creation date)."""

    return render_template(
        'post_list.html',
        title=u'Posts',
        posts=current_app.hg.get_all(True),
    )


@views.route('/tag/<path:tag>/')
def tag(tag):
    """Page that lists the abstract of all available posts for the given
    tag.
    """

    tags = tag.split('/')
    for _tag in tags:
        if _tag not in current_app.hg.tags:
            abort(404)
    posts = current_app.hg.get_by_tag(tags)
    return render_template(
        '_posts.html',
        title=u'Tag: %s' % ' + '.join(tags),
        tag=tags,
        posts=posts,
        full_content=False,
    )


@views.route('/source/')  # just to make robots.txt's url_for happy :)
@views.route('/source/<path:slug>/')
def source(slug=None):
    """View that shows the source code of a given static page/post."""

    if slug is None:
        abort(404)
    source = current_app.hg.get(slug)
    if source is None:
        abort(404)
    response = make_response(source.full)
    response.headers['Content-Type'] = 'text/plain; charset=utf-8'
    return response


@views.route('/robots.txt')
def robots():
    """Viev that generates a robots.txt file for ``/source/``  path, to avoid
    source files to be indexed by search engines. Can be disabled setting
    ROBOTS_TXT configuration parameter to ``False``
    """

    if not current_app.config['ROBOTS_TXT']:
        abort(404)
    response = make_response(render_template('robots.txt'))
    response.headers['Content-Type'] = 'text/plain; charset=utf-8'
    return response



# -*- coding: utf-8 -*-
"""
    blohg.mercurial_theme
    ~~~~~~~~~~~~~~~~~~~~~

    Module with the stuff needed by blohg to use Jinja2 templates and
    static files from the Mercurial repository.

    :copyright: (c) 2011 by Rafael Goncalves Martins
    :license: GPL-2, see LICENSE for more details.
"""

from flask import current_app, g, request, abort
from jinja2.loaders import BaseLoader, ChoiceLoader, TemplateNotFound, \
    split_template_path
from time import time
from zlib import adler32

import posixpath
import mimetypes


def setup_theme(app):
    """This function replaces the default jinja loader from flask with
    a ChoiceLoader, that tries to load templates from the mercurial
    repository and fallback to the default jinja loader. This function
    also replace the static url endpoint by another, that load files
    from the mercurial repository as well.

    :param app: the application object.
    """

    old_loader = app.jinja_loader
    app.jinja_loader = ChoiceLoader([
        MercurialLoader(),
        old_loader,
    ])
    app.add_url_rule(
        app.static_path + '/<path:filename>',
        endpoint='static',
        view_func=MercurialStaticFile(app.config['STATIC_DIR'])
    )
    app.add_url_rule(
        '/attachments/<path:filename>',
        endpoint='attachments',
        view_func=MercurialStaticFile(app.config['ATTACHMENT_DIR'])
    )


class MercurialLoader(BaseLoader):
    """A Jinja2 loader that loads templates from a Mercurial repository"""

    def get_source(self, environment, template):
        pieces = split_template_path(template)
        templates_dir = current_app.config['TEMPLATES_DIR']
        filename = posixpath.join(templates_dir, *pieces)
        if filename in list(current_app.hg.revision):
            contents = current_app.hg.revision[filename].data().decode('utf-8')
            return contents, filename, lambda: not g.refresh
        raise TemplateNotFound(template)

    def list_templates(self):
        templates_dir = current_app.config['TEMPLATES_DIR']
        return sorted([i for i in current_app.hg.revision \
            if i.startswith(templates_dir + '/')])


class MercurialStaticFile(object):
    """Callable to create a Response object for static files loaded from
    the current Mercurial repository.
    """

    def __init__(self, directory):
        self._directory = directory

    def __call__(self, filename):
        filename = posixpath.join(self._directory, filename)
        mimetype = mimetypes.guess_type(filename)[0]
        if mimetype is None:
            mimetype = 'application/octet-stream'
        try:
            data = current_app.hg.revision[filename].data()
        except:
            abort(404)
        rv = current_app.response_class(data, mimetype=mimetype,
            direct_passthrough=True)
        rv.cache_control.public = True
        cache_timeout = 60 * 60 * 12
        rv.cache_control.max_age = cache_timeout
        rv.expires = int(time() + cache_timeout)
        try:
            date = int(current_app.hg.revision[filename].date()[0])
        except:
            date = time()
        rv.set_etag('blohg-%s-%s-%s' % (date, len(data), adler32(filename) \
            & 0xffffffff))
        rv = rv.make_conditional(request)
        return rv



# -*- coding: utf-8 -*-
"""
    blohg.version
    ~~~~~~~~~~~~~

    Module with the version number related stuff.

    :copyright: (c) 2010-2011 by Rafael Goncalves Martins
    :license: GPL-2, see LICENSE for more details.
"""

# CHANGE ME BEFORE THE RELEASE!
version = '0.5.1'

# code snippet from sphinx
# http://bitbucket.org/birkenfeld/sphinx/src/tip/sphinx/__init__.py
if '+' in version or 'pre' in version:
    # try to find out the changeset hash if checked out from hg, and append
    # it to version (since we use this value from setup.py, it gets
    # automatically propagated to an installed copy as well)
    try:
        import os
        import subprocess
        cwd = os.path.dirname(os.path.abspath(__file__))
        p = subprocess.Popen(
            ['hg', 'id', '-i', '-R', os.path.join(cwd, '..')],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE
        )
        out, err = p.communicate()
        if out:
            version += '/' + out.strip()
    except Exception:
        pass



# -*- coding: utf-8 -*-
"""
    blohg.rst_roles
    ~~~~~~~~~~~~~~~

    Module with the custom blohg reStructuredText roles.

    :copyright: (c) 2011 by Rafael Goncalves Martins
    :license: GPL-2, see LICENSE for more details.
"""

from docutils.nodes import reference
from flask import current_app, url_for

import posixpath


__all__ = ['attachment_role']


def attachment_role(name, rawtext, text, lineno, inliner, options={}, content=[]):
    """reStructuredText role that creates a Text node with the full url for an
    attachment.
    """

    full_path = posixpath.join(current_app.config['ATTACHMENT_DIR'], text)
    if full_path not in list(current_app.hg.revision):
        msg = inliner.reporter.error(
            'Error in "%s" role: File not found: %s.' % (
                name, full_path
            ),
            line=lineno
        )
        prb = inliner.problematic(rawtext, rawtext, msg)
        return [prb], [msg]
    url = url_for('.attachments', filename=text, _external=True)
    return [reference(url, url, refuri=url)], []


__roles__ = {
    'attachment': attachment_role,
}



# -*- coding: utf-8 -*-
"""
    blohg.mercurial_content
    ~~~~~~~~~~~~~~~~~~~~~~~

    Module with Mercurial-related stuff needed by blohg to load posts
    and static pages.

    :copyright: (c) 2010-2011 by Rafael Goncalves Martins
    :license: GPL-2, see LICENSE for more details.
"""

import os
import re
import time
import yaml

from datetime import datetime
from docutils.core import publish_parts
from docutils.parsers.rst.directives import register_directive
from docutils.parsers.rst.roles import register_local_role
from mercurial import hg, ui
from werkzeug.utils import cached_property

from blohg import rst_directives, rst_roles

try:
    from flask import current_app, g
except ImportError:
    current_app = None
    g = None

re_metadata = re.compile(r'\.\. +([a-z]*): (.*)')
re_read_more = re.compile(r'\.\. +read_more')


# registering docutils' directives
for directive in rst_directives.__directives__:
    register_directive(directive, rst_directives.__directives__[directive])

# registering docutils' roles
for role in rst_roles.__roles__:
    register_local_role(role, rst_roles.__roles__[role])


def rst2html(rst):
    """Function that converts reStructuredText to HTML, returning the body
    of the HTML file.

    :param rst: the reStructuredText string.
    :return: a dict with the title and the fragment of the generated HTML.
    """

    parts = publish_parts(
        source=rst,
        writer_name='html4css1',
        settings_overrides={
            'input_encoding': 'utf-8',
            'output_encoding': 'utf-8',
            'initial_header_level': 3,
        }
    )
    return {
        'title': parts['title'],
        'fragment': parts['fragment'],
    }


def load_config(app):
    """This function merges the configuration parameters from the YAML
    file with the ``app.config`` object.

    :param app: the application object.
    """

    app.config.update(yaml.load(app.hg.config))

    # debug parameters
    if app.debug:
        if 'GOOGLE_ANALYTICS' in app.config:
            del app.config['GOOGLE_ANALYTICS']
        app.config['DISQUS_DEVELOPER'] = True


def setup_mercurial(app):
    """This function adds a :class:`MercurialContent` instance to an
    application object, as a ``hg`` attribute, and reloads it as needed.

    :param app: the application object.
    """

    @app.before_request
    def before_request():

        # always init the repository
        repo_path = app.config.get('REPO_PATH', '.')
        repo = hg.repository(ui.ui(), repo_path)

        refresh = False

        if app.debug:
            refresh = True

        if not hasattr(app, 'hg'):
            refresh = True

        # If we still don't want a refresh, it seems that we have an app.hg
        # object, then we need to check if it's new enough
        if not refresh:
            # using the 'tip' revision, as we're not debugging, and just
            # want the committed stuff.
            if repo['tip'].rev() != app.hg.repo['tip'].rev():
                refresh = True

        if g is not None:
            g.refresh = refresh

        # refreshing :)
        if refresh:

            # Deleting the app.hg element. Not sure if it's worth :)
            if hasattr(app, 'hg'):
                del app.hg

            revision_id = None
            if not app.debug:
                revision_id = 'tip'
            app.hg = MercurialContent(repo, revision_id)
            load_config(app)

    @app.after_request
    def after_request(response):
        if g is not None:
            g.refresh = False
        return response


class MercurialContent(object):
    """Object that represents a blohg Mercurial repository."""

    config_file = 'config.yaml'
    content_dir = 'content'

    def __init__(self, repo, revision_id):
        """Class constructor"""

        self.repo = repo
        self.revision_id = revision_id
        self.revision = repo[revision_id]

        if current_app is not None:
            self.content_dir = current_app.config['CONTENT_DIR']

    @cached_property
    def _pages(self):
        """Method that returns an ordered list with all the pages/posts
        available.

        :return: a list of :class:`Metadata` objects.
        """

        metadata = []
        for filename in self.revision:
            if filename.startswith(self.content_dir + '/') and \
               filename.endswith('.rst'):
                metadata.append(Metadata(self.repo, self.revision[filename]))
        return sorted(metadata, self._compare_by_date)

    def _compare_by_date(self, a, b):
        """Method that compares 2 :class:`Metadata` objects by creation
        date.
        """

        return b.date - a.date

    def get(self, filename):
        """Method that returns a :class:`Metadata` object for a given
        filename.

        :param filename: the file name string.
        :return: a :class:`Metadata` object.
        """

        full_path = os.path.join(self.content_dir, filename + '.rst')
        for page in self._pages:
            if page.path == full_path:
                return page
        return None

    def get_all(self, only_posts=False):
        """Method that returns a list of :class:`Metadata` objects for
        all the available files, ordered by creation date.

        :param only_posts: a boolean that makes the method returns only
                           the available posts for the given locale, not
                           the static pages.
        :return: a list of :class:`Metadata` objects.
        """

        my_posts_dir = os.path.join(self.content_dir, 'post')
        metadata = []
        for page in self._pages:
            if page.path.startswith(self.content_dir) and not only_posts:
                metadata.append(page)
                continue
            if page.path.startswith(my_posts_dir):
                metadata.append(page)
        return metadata

    def get_by_tag(self, tag):
        """Method that returns a list of :class:`Metadata` objects for a
        given list of tag identifier strings.

        :param tag: a list of tag identifier strings.
        :return: a list of :class:`Metadata` objects.
        """

        posts = []
        for post in self.get_all(only_posts=True):
            valid = True
            for _tag in tag:
                if _tag not in post.tags:
                    valid = False
            if valid:
                posts.append(post)
        return posts

    @cached_property
    def tags(self):
        """Method that returns a list of all the available tag identifiers.

        :return: a list of tag identifiers strings.
        """

        tags = []
        for post in self.get_all(only_posts=True):
            for tag in post.tags:
                if tag not in tags:
                    tags.append(tag)
        tags.sort()
        return tags

    @cached_property
    def config(self):
        """Method that returns a string with the content of the config.yaml
        file.

        :return: a string with the content of the configuration file.
        """

        if self.config_file not in list(self.revision):
            raise RuntimeError('Configuration file not found: %r' % \
                self.config_file)
        return self.revision[self.config_file].data()

    def __repr__(self):
        return '<MercurialContent %r>' % self.repo.root


class Metadata(object):
    """Static page/blog post metadata object."""

    _title = None
    _content_dir = 'content'

    def __init__(self, repo, filectx):
        """Class constructor.

        :param repo: a Mercurial repository object (not
                     :class:`MercurialContent`)
        :param filectx: the Mercurial file context of the file.
        """

        if current_app is not None:
            self._content_dir = current_app.config['CONTENT_DIR']

        self._repo = repo
        self._filectx = filectx
        self._filecontent = filectx.data()
        self._vars = {}
        for i in re_metadata.finditer(self._filecontent):
            self._vars[i.group(1)] = i.group(2).decode('utf-8')
        if 'tags' in self._vars:
            self._vars['tags'] = [i.strip() for i in \
                                  self._vars['tags'].split(',')]
        filelog = self._filectx.filelog()
        changesets = list(filelog)
        if 'date' in self._vars:
            try:
                self._vars['date'] = int(self._vars['date'])
            except ValueError:
                del self._vars['date']
        else:
            first_changeset = self._repo[filelog.linkrev(0)]
            self._vars['date'] = int(first_changeset.date()[0])
            if self._vars['date'] == 0:
                self._vars['date'] = int(time.time())
        self._vars['datetime'] = datetime.utcfromtimestamp(self._vars['date'])
        if 'mdate' in self._vars:
            try:
                self._vars['mdate'] = int(self._vars['mdate'])
            except ValueError:
                del self._vars['mdate']
        if 'mdate' not in self._vars and len(changesets) > 1:
            last_changeset = self._repo[filelog.linkrev(len(changesets) - 1)]
            self._vars['mdate'] = int(last_changeset.date()[0])
        if 'mdate' in self._vars:
            self._vars['mdatetime'] = \
                datetime.utcfromtimestamp(self._vars['mdate'])

    def _set_title(self, title):
        if self._title is None:
            title = title.strip()
            if title != '':
                self._title = title

    @cached_property
    def path(self):
        return self._filectx.path()

    @cached_property
    def slug(self):
        match = re.match(r'%s/(.+)\.rst' % self._content_dir,
            self._filectx.path())
        if match is not None:
            return match.group(1)

    @cached_property
    def title(self):
        # workaround to load the title from rst
        self.abstract_html
        if self._title is not None:
            return self._title
        return self._vars.get('title', u'')

    @cached_property
    def tags(self):
        return self._vars.get('tags', [])

    @cached_property
    def abstract(self):
        return re_read_more.split(self._filecontent)[0].decode('utf-8')

    @cached_property
    def abstract_html(self):
        parts = rst2html(self.abstract)
        self._set_title(parts['title'])
        return parts['fragment']

    @cached_property
    def full(self):
        return self._filecontent.decode('utf-8')

    @cached_property
    def full_html(self):
        parts = rst2html(self.full)
        self._set_title(parts['title'])
        return parts['fragment']

    @cached_property
    def read_more(self):
        return len(re_read_more.split(self._filecontent)) > 1

    def get(self, key, default=None):
        return self._vars.get(key, default)

    def __getattr__(self, attr):
        return self.get(attr)

    def __str__(self):
        return self._filecontent

    def __repr__(self):
        return '<Metadata %r>' % self.slug



# -*- coding: utf-8 -*-
"""
    blohg.rst_directives
    ~~~~~~~~~~~~~~~~~~~~

    Module with the custom blohg reStructuredText directives.

    :copyright: (c) 2010-2011 by Rafael Goncalves Martins
    :license: GPL-2, see LICENSE for more details.
"""

from docutils import nodes
from docutils.parsers.rst import directives, Directive
from docutils.parsers.rst.directives.images import Image, Figure
from flask import current_app, request, url_for
from pygments import highlight
from pygments.formatters import HtmlFormatter
from pygments.lexers import get_lexer_by_name, TextLexer
from urllib import pathname2url

import posixpath


__all__ = ['Youtube', 'Math', 'Code', 'SourceCode', 'AttachmentImage',
           'AttachmentFigure', 'SubPages']

GOOGLETEX_URL = 'https://chart.googleapis.com/chart?cht=tx&chl='


def align(argument):
    return directives.choice(argument, ('left', 'center', 'right'))


class Youtube(Directive):
    """reStructuredText directive that creates an embed object to display
    a video from Youtube

    Usage example::

        .. youtube:: QFwQIRwuAM0
           :align: center
           :height: 344
           :width: 425
    """

    required_arguments = 1
    optional_arguments = 0
    option_spec = {
        'height': directives.nonnegative_int,
        'width': directives.nonnegative_int,
        'align': align,
    }
    has_content = False

    def run(self):
        self.options['vid'] = self.arguments[0]
        if not 'width' in self.options:
            self.options['width'] = 425
        if not 'height' in self.options:
            self.options['height'] = 344
        if not 'align' in self.options:
            self.options['align'] = 'center'
        html = '''\
<div align="%(align)s">
    <iframe width="%(width)i" height="%(height)i" src="http://www.youtube.com/embed/%(vid)s" frameborder="0" allowfullscreen></iframe>
</div>
'''
        return [nodes.raw('', html % self.options, format='html')]


class Code(Directive):
    """reStructuredText directive that creates a pre tag suitable for
    decoration with http://alexgorbatchev.com/SyntaxHighlighter/

    Usage example::

        .. source:: python

           print "Hello, World!"

        .. raw:: html

            <script type="text/javascript" src="http://alexgorbatchev.com/pub/sh/current/scripts/shCore.js"></script>
            <script type="text/javascript" src="http://alexgorbatchev.com/pub/sh/current/scripts/shBrushPython.js"></script>
            <link type="text/css" rel="stylesheet" href="http://alexgorbatchev.com/pub/sh/current/styles/shCoreDefault.css"/>
            <script type="text/javascript">SyntaxHighlighter.defaults.toolbar=false; SyntaxHighlighter.all();</script>

    """

    required_arguments = 1
    optional_arguments = 0
    has_content = True

    def run(self):
        self.assert_has_content()
        self.options['brush'] = self.arguments[0]
        html = '''\

<pre class="brush: %s">
%s
</pre>

'''
        return [nodes.raw('', html % (self.options['brush'],
            "\n".join(self.content).replace('<', '&lt;')),
            format='html')]


class SourceCode(Directive):
    """reStructuredText directive that does syntax highlight using Pygments.

    Usage example::

        .. sourcecode:: python
           :linenos:

            print "Hello, World!"

    The ``linenos`` option enables the line numbering.

    To be able to use this directive you should generate a CSS file with the style
    definitions, using the ``pygmentize`` script, shipped with Pygments.

    ::

        $ pygmentyze -S friendly -f html > static/pygments.css

    Where ``friendly`` will be your Pygments style of choice.

    This file should be included in the main template, usually ``base.html``::

        <link type="text/css" media="screen" rel="stylesheet" href="{{
            url_for('.static', filename='pygments.css') }}" />

    This directive is based on ``rst-directive.py``, created by Pygments authors.
    """

    required_arguments = 1
    optional_arguments = 0
    final_argument_whitespace = True
    option_spec = {
        'linenos': directives.flag,
    }
    has_content = True

    def run(self):
        self.assert_has_content()
        try:
            lexer = get_lexer_by_name(self.arguments[0])
        except ValueError:
            # no lexer found - use the text one instead of an exception
            lexer = TextLexer()
        formatter = HtmlFormatter(noclasses=False)
        if 'linenos' in self.options:
            formatter.linenos = 2  # inline
        parsed = highlight(u'\n'.join(self.content), lexer, formatter)
        return [nodes.raw('', parsed, format='html')]


class Math(Image):
    """reStructuredText directive that creates an image HTML object to
    display a LaTeX equation, using Google Chart API.

    Usage example::

        .. math::

            \frac{x^2}{1+x}
    """

    required_arguments = 0
    has_content = True

    def run(self):
        self.assert_has_content()
        if not 'align' in self.options:
            self.options['align'] = 'center'
        tmp = pathname2url(' '.join([(i == '' and '\\\\' or i.strip()) \
            for i in self.content]))
        self.arguments.append('%s%s' % (GOOGLETEX_URL, tmp))
        return Image.run(self)


class AttachmentImage(Image):

    def run(self):
        my_file = directives.uri(self.arguments[0])
        full_path = posixpath.join(current_app.config['ATTACHMENT_DIR'], my_file)
        if full_path not in list(current_app.hg.revision):
            raise self.error(
                'Error in "%s" directive: File not found: %s.' % (
                    self.name, full_path
                )
            )
        self.arguments[0] = url_for('.attachments', filename=my_file, _external=True)
        return Image.run(self)


class AttachmentFigure(Figure):

    def run(self):
        my_file = directives.uri(self.arguments[0])
        full_path = posixpath.join(current_app.config['ATTACHMENT_DIR'], my_file)
        if full_path not in list(current_app.hg.revision):
            raise self.error(
                'Error in "%s" directive: File not found: %s.' % (
                    self.name, full_path
                )
            )
        self.arguments[0] = url_for('.attachments', filename=my_file, _external=True)
        return Figure.run(self)


class SubPages(Directive):
    """reStructuredText directive that creates a bullet-list with the subpages of
    the current page, or of a given page.

    Usage example::

        .. subpages::

    Or::

        .. subpages:: projects

    Supposing that you have a directory called ``content/projects`` and some
    reStructuredText files on it. Subdirectories are also allowed.

    This directive will just show the files from the root of the directory. It's
    not recursive.
    """

    required_arguments = 0
    optional_arguments = 1
    has_content = False

    def _compare_by_slug(self, x, y):
        if x.slug > y.slug:
            return 1
        if x.slug < y.slug:
            return -1
        return 0

    def run(self):
        if len(self.arguments) == 0:
            self.arguments.append(request.path.strip('/'))
        tmp_metadata = []
        final_metadata = []
        # not sure if this works on windows
        splited_dir = self.arguments[0].split('/')
        for metadata in current_app.hg.get_all():
            # not sure if this works on windows
            splited_slug = metadata.slug.split('/')
            if metadata.slug.startswith(self.arguments[0]) and \
               (len(splited_dir) + 1 == len(splited_slug)):
                tmp_metadata.append(metadata)
        for metadata in sorted(tmp_metadata, self._compare_by_slug):
            link = url_for('views.content', slug=metadata.slug)
            reference = nodes.reference(link, metadata.title, refuri=link)
            final_metadata.append(nodes.list_item('', nodes.paragraph('', '', reference)))
        return [nodes.bullet_list('', *final_metadata)]


__directives__ = {
    'youtube': Youtube,
    'math': Math,
    'code': Code,
    'sourcecode': SourceCode,
    'attachment-image': AttachmentImage,
    'attachment-figure': AttachmentFigure,
    'subpages': SubPages,
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
    setup.py
    ~~~~~~~~

    :copyright: (c) 2010-2011 by Rafael Goncalves Martins
    :license: GPL-2, see LICENSE for more details.
"""

from setuptools import setup, find_packages

# doing things the wrong way...
# we need the module blohg.version but we can't import the full package
# first time because the dependencies probably aren't solved yet.
import os
import sys
cwd = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(cwd, 'blohg'))
from version import version as __version__

setup(
    name='blohg',
    version=__version__,
    license='GPL-2',
    description='A Mercurial-based blog engine',
    long_description=open(os.path.join(cwd, 'README.rst')).read(),
    author='Rafael Goncalves Martins',
    author_email='rafael@rafaelmartins.eng.br',
    url='http://blohg.org/',
    platforms='any',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'docutils>=0.7',
        'Flask>=0.6',
        'Flask-Babel>=0.6',
        'Flask-Script>=0.3',
        'Jinja2>=2.5.2',
        'Mercurial>=1.6',
        'PyYAML',
        'Pygments',  # TODO: make this dependency optional
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content :: News/Diary',
        'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',
        'Topic :: Software Development :: Version Control',
    ],
    data_files=[
        ('share/blohg', ['share/blohg.wsgi']),
    ],
    scripts=['bin/blohg'],
)
